package edu.umontreal.kotlingrad.dependent

open class Mat<T, Rows : D100, Cols : D100> : Vec<Vec<T, Cols>, Rows> {
  /**TODO: Make contents a Vec<Vec<T, MaxCols>, MaxRows>**/
  val rowT: Nat<Rows>
  val colT: Nat<Cols>
  val vec: Vec<Vec<T, Cols>, Rows>

  constructor(rowT: Nat<Rows>, colT: Nat<Cols>, vec: Vec<Vec<T, Cols>, Rows>) : super(rowT, vec.contents) {
    this.rowT = rowT
    this.colT = colT
    this.vec = vec
    this.numRows = rowT.i
    this.numCols = colT.i
  }

  val numRows: Int
  val numCols: Int

  constructor(rowT: Nat<Rows>, colT: Nat<Cols>, filler: T) : this(rowT, colT,
    Vec(rowT, (0 until rowT.i).map { Vec(colT, (0 until colT.i).map { filler }) }))

  constructor(rowT: Nat<Rows>, colT: Nat<Cols>, filler: () -> T) : this(rowT, colT,
    Vec(rowT, (0 until rowT.i).map { Vec(colT, (0 until colT.i).map { filler() }) }))

  constructor(rowT: Nat<Rows>, colT: Nat<Cols>, filler: (row: Int, col: Int) -> T) : this(rowT, colT,
    Vec(rowT, (0 until rowT.i).map { row -> Vec(colT, (0 until colT.i).map { col -> filler(row, col) }) }))

  constructor(rowT: Nat<Rows>, colT: Nat<Cols>, ts: List<T>) : this(rowT, colT,
    Vec(rowT, (0 until rowT.i).map { row -> Vec(colT, (0 until colT.i).map { col -> ts[col + row * colT.i] }) }.toList()))

  override fun toString() = "($numRows x $numCols)\n[${contents.joinToString("\n ") { it.contents.joinToString(" ") }}]"

  // Type-safe construction for matrix literals up to 9x9
  @Suppress("UNUSED_PARAMETER")
  companion object {
    @JvmName("m1x1") private operator fun <T, V: Vec<T, D1>> invoke(t0: V): Mat<T, D1, D1> = Mat(D1, D1, Vec(t0))
    @JvmName("m1x2") private operator fun <T, V: Vec<T, D2>> invoke(t0: V): Mat<T, D1, D2> = Mat(D1, D2, Vec(t0))
    @JvmName("m1x3") private operator fun <T, V: Vec<T, D3>> invoke(t0: V): Mat<T, D1, D3> = Mat(D1, D3, Vec(t0))
    @JvmName("m1x4") private operator fun <T, V: Vec<T, D4>> invoke(t0: V): Mat<T, D1, D4> = Mat(D1, D4, Vec(t0))
    @JvmName("m1x5") private operator fun <T, V: Vec<T, D5>> invoke(t0: V): Mat<T, D1, D5> = Mat(D1, D5, Vec(t0))
    @JvmName("m1x6") private operator fun <T, V: Vec<T, D6>> invoke(t0: V): Mat<T, D1, D6> = Mat(D1, D6, Vec(t0))
    @JvmName("m1x7") private operator fun <T, V: Vec<T, D7>> invoke(t0: V): Mat<T, D1, D7> = Mat(D1, D7, Vec(t0))
    @JvmName("m1x8") private operator fun <T, V: Vec<T, D8>> invoke(t0: V): Mat<T, D1, D8> = Mat(D1, D8, Vec(t0))
    @JvmName("m1x9") private operator fun <T, V: Vec<T, D9>> invoke(t0: V): Mat<T, D1, D9> = Mat(D1, D9, Vec(t0))
    @JvmName("m2x1") private operator fun <T, V: Vec<T, D1>> invoke(t0: V, t1: V): Mat<T, D2, D1> = Mat(D2, D1, Vec(t0, t1))
    @JvmName("m2x2") private operator fun <T, V: Vec<T, D2>> invoke(t0: V, t1: V): Mat<T, D2, D2> = Mat(D2, D2, Vec(t0, t1))
    @JvmName("m2x3") private operator fun <T, V: Vec<T, D3>> invoke(t0: V, t1: V): Mat<T, D2, D3> = Mat(D2, D3, Vec(t0, t1))
    @JvmName("m2x4") private operator fun <T, V: Vec<T, D4>> invoke(t0: V, t1: V): Mat<T, D2, D4> = Mat(D2, D4, Vec(t0, t1))
    @JvmName("m2x5") private operator fun <T, V: Vec<T, D5>> invoke(t0: V, t1: V): Mat<T, D2, D5> = Mat(D2, D5, Vec(t0, t1))
    @JvmName("m2x6") private operator fun <T, V: Vec<T, D6>> invoke(t0: V, t1: V): Mat<T, D2, D6> = Mat(D2, D6, Vec(t0, t1))
    @JvmName("m2x7") private operator fun <T, V: Vec<T, D7>> invoke(t0: V, t1: V): Mat<T, D2, D7> = Mat(D2, D7, Vec(t0, t1))
    @JvmName("m2x8") private operator fun <T, V: Vec<T, D8>> invoke(t0: V, t1: V): Mat<T, D2, D8> = Mat(D2, D8, Vec(t0, t1))
    @JvmName("m2x9") private operator fun <T, V: Vec<T, D9>> invoke(t0: V, t1: V): Mat<T, D2, D9> = Mat(D2, D9, Vec(t0, t1))
    @JvmName("m3x1") private operator fun <T, V: Vec<T, D1>> invoke(t0: V, t1: V, t2: V): Mat<T, D3, D1> = Mat(D3, D1, Vec(t0, t1, t2))
    @JvmName("m3x2") private operator fun <T, V: Vec<T, D2>> invoke(t0: V, t1: V, t2: V): Mat<T, D3, D2> = Mat(D3, D2, Vec(t0, t1, t2))
    @JvmName("m3x3") private operator fun <T, V: Vec<T, D3>> invoke(t0: V, t1: V, t2: V): Mat<T, D3, D3> = Mat(D3, D3, Vec(t0, t1, t2))
    @JvmName("m3x4") private operator fun <T, V: Vec<T, D4>> invoke(t0: V, t1: V, t2: V): Mat<T, D3, D4> = Mat(D3, D4, Vec(t0, t1, t2))
    @JvmName("m3x5") private operator fun <T, V: Vec<T, D5>> invoke(t0: V, t1: V, t2: V): Mat<T, D3, D5> = Mat(D3, D5, Vec(t0, t1, t2))
    @JvmName("m3x6") private operator fun <T, V: Vec<T, D6>> invoke(t0: V, t1: V, t2: V): Mat<T, D3, D6> = Mat(D3, D6, Vec(t0, t1, t2))
    @JvmName("m3x7") private operator fun <T, V: Vec<T, D7>> invoke(t0: V, t1: V, t2: V): Mat<T, D3, D7> = Mat(D3, D7, Vec(t0, t1, t2))
    @JvmName("m3x8") private operator fun <T, V: Vec<T, D8>> invoke(t0: V, t1: V, t2: V): Mat<T, D3, D8> = Mat(D3, D8, Vec(t0, t1, t2))
    @JvmName("m3x9") private operator fun <T, V: Vec<T, D9>> invoke(t0: V, t1: V, t2: V): Mat<T, D3, D9> = Mat(D3, D9, Vec(t0, t1, t2))
    @JvmName("m4x1") private operator fun <T, V: Vec<T, D1>> invoke(t0: V, t1: V, t2: V, t3: V): Mat<T, D4, D1> = Mat(D4, D1, Vec(t0, t1, t2, t3))
    @JvmName("m4x2") private operator fun <T, V: Vec<T, D2>> invoke(t0: V, t1: V, t2: V, t3: V): Mat<T, D4, D2> = Mat(D4, D2, Vec(t0, t1, t2, t3))
    @JvmName("m4x3") private operator fun <T, V: Vec<T, D3>> invoke(t0: V, t1: V, t2: V, t3: V): Mat<T, D4, D3> = Mat(D4, D3, Vec(t0, t1, t2, t3))
    @JvmName("m4x4") private operator fun <T, V: Vec<T, D4>> invoke(t0: V, t1: V, t2: V, t3: V): Mat<T, D4, D4> = Mat(D4, D4, Vec(t0, t1, t2, t3))
    @JvmName("m4x5") private operator fun <T, V: Vec<T, D5>> invoke(t0: V, t1: V, t2: V, t3: V): Mat<T, D4, D5> = Mat(D4, D5, Vec(t0, t1, t2, t3))
    @JvmName("m4x6") private operator fun <T, V: Vec<T, D6>> invoke(t0: V, t1: V, t2: V, t3: V): Mat<T, D4, D6> = Mat(D4, D6, Vec(t0, t1, t2, t3))
    @JvmName("m4x7") private operator fun <T, V: Vec<T, D7>> invoke(t0: V, t1: V, t2: V, t3: V): Mat<T, D4, D7> = Mat(D4, D7, Vec(t0, t1, t2, t3))
    @JvmName("m4x8") private operator fun <T, V: Vec<T, D8>> invoke(t0: V, t1: V, t2: V, t3: V): Mat<T, D4, D8> = Mat(D4, D8, Vec(t0, t1, t2, t3))
    @JvmName("m4x9") private operator fun <T, V: Vec<T, D9>> invoke(t0: V, t1: V, t2: V, t3: V): Mat<T, D4, D9> = Mat(D4, D9, Vec(t0, t1, t2, t3))
    @JvmName("m5x1") private operator fun <T, V: Vec<T, D1>> invoke(t0: V, t1: V, t2: V, t3: V, t4: V): Mat<T, D5, D1> = Mat(D5, D1, Vec(t0, t1, t2, t3, t4))
    @JvmName("m5x2") private operator fun <T, V: Vec<T, D2>> invoke(t0: V, t1: V, t2: V, t3: V, t4: V): Mat<T, D5, D2> = Mat(D5, D2, Vec(t0, t1, t2, t3, t4))
    @JvmName("m5x3") private operator fun <T, V: Vec<T, D3>> invoke(t0: V, t1: V, t2: V, t3: V, t4: V): Mat<T, D5, D3> = Mat(D5, D3, Vec(t0, t1, t2, t3, t4))
    @JvmName("m5x4") private operator fun <T, V: Vec<T, D4>> invoke(t0: V, t1: V, t2: V, t3: V, t4: V): Mat<T, D5, D4> = Mat(D5, D4, Vec(t0, t1, t2, t3, t4))
    @JvmName("m5x5") private operator fun <T, V: Vec<T, D5>> invoke(t0: V, t1: V, t2: V, t3: V, t4: V): Mat<T, D5, D5> = Mat(D5, D5, Vec(t0, t1, t2, t3, t4))
    @JvmName("m5x6") private operator fun <T, V: Vec<T, D6>> invoke(t0: V, t1: V, t2: V, t3: V, t4: V): Mat<T, D5, D6> = Mat(D5, D6, Vec(t0, t1, t2, t3, t4))
    @JvmName("m5x7") private operator fun <T, V: Vec<T, D7>> invoke(t0: V, t1: V, t2: V, t3: V, t4: V): Mat<T, D5, D7> = Mat(D5, D7, Vec(t0, t1, t2, t3, t4))
    @JvmName("m5x8") private operator fun <T, V: Vec<T, D8>> invoke(t0: V, t1: V, t2: V, t3: V, t4: V): Mat<T, D5, D8> = Mat(D5, D8, Vec(t0, t1, t2, t3, t4))
    @JvmName("m5x9") private operator fun <T, V: Vec<T, D9>> invoke(t0: V, t1: V, t2: V, t3: V, t4: V): Mat<T, D5, D9> = Mat(D5, D9, Vec(t0, t1, t2, t3, t4))
    @JvmName("m6x1") private operator fun <T, V: Vec<T, D1>> invoke(t0: V, t1: V, t2: V, t3: V, t4: V, t5: V): Mat<T, D6, D1> = Mat(D6, D1, Vec(t0, t1, t2, t3, t4, t5))
    @JvmName("m6x2") private operator fun <T, V: Vec<T, D2>> invoke(t0: V, t1: V, t2: V, t3: V, t4: V, t5: V): Mat<T, D6, D2> = Mat(D6, D2, Vec(t0, t1, t2, t3, t4, t5))
    @JvmName("m6x3") private operator fun <T, V: Vec<T, D3>> invoke(t0: V, t1: V, t2: V, t3: V, t4: V, t5: V): Mat<T, D6, D3> = Mat(D6, D3, Vec(t0, t1, t2, t3, t4, t5))
    @JvmName("m6x4") private operator fun <T, V: Vec<T, D4>> invoke(t0: V, t1: V, t2: V, t3: V, t4: V, t5: V): Mat<T, D6, D4> = Mat(D6, D4, Vec(t0, t1, t2, t3, t4, t5))
    @JvmName("m6x5") private operator fun <T, V: Vec<T, D5>> invoke(t0: V, t1: V, t2: V, t3: V, t4: V, t5: V): Mat<T, D6, D5> = Mat(D6, D5, Vec(t0, t1, t2, t3, t4, t5))
    @JvmName("m6x6") private operator fun <T, V: Vec<T, D6>> invoke(t0: V, t1: V, t2: V, t3: V, t4: V, t5: V): Mat<T, D6, D6> = Mat(D6, D6, Vec(t0, t1, t2, t3, t4, t5))
    @JvmName("m6x7") private operator fun <T, V: Vec<T, D7>> invoke(t0: V, t1: V, t2: V, t3: V, t4: V, t5: V): Mat<T, D6, D7> = Mat(D6, D7, Vec(t0, t1, t2, t3, t4, t5))
    @JvmName("m6x8") private operator fun <T, V: Vec<T, D8>> invoke(t0: V, t1: V, t2: V, t3: V, t4: V, t5: V): Mat<T, D6, D8> = Mat(D6, D8, Vec(t0, t1, t2, t3, t4, t5))
    @JvmName("m6x9") private operator fun <T, V: Vec<T, D9>> invoke(t0: V, t1: V, t2: V, t3: V, t4: V, t5: V): Mat<T, D6, D9> = Mat(D6, D9, Vec(t0, t1, t2, t3, t4, t5))
    @JvmName("m7x1") private operator fun <T, V: Vec<T, D1>> invoke(t0: V, t1: V, t2: V, t3: V, t4: V, t5: V, t6: V): Mat<T, D7, D1> = Mat(D7, D1, Vec(t0, t1, t2, t3, t4, t5, t6))
    @JvmName("m7x2") private operator fun <T, V: Vec<T, D2>> invoke(t0: V, t1: V, t2: V, t3: V, t4: V, t5: V, t6: V): Mat<T, D7, D2> = Mat(D7, D2, Vec(t0, t1, t2, t3, t4, t5, t6))
    @JvmName("m7x3") private operator fun <T, V: Vec<T, D3>> invoke(t0: V, t1: V, t2: V, t3: V, t4: V, t5: V, t6: V): Mat<T, D7, D3> = Mat(D7, D3, Vec(t0, t1, t2, t3, t4, t5, t6))
    @JvmName("m7x4") private operator fun <T, V: Vec<T, D4>> invoke(t0: V, t1: V, t2: V, t3: V, t4: V, t5: V, t6: V): Mat<T, D7, D4> = Mat(D7, D4, Vec(t0, t1, t2, t3, t4, t5, t6))
    @JvmName("m7x5") private operator fun <T, V: Vec<T, D5>> invoke(t0: V, t1: V, t2: V, t3: V, t4: V, t5: V, t6: V): Mat<T, D7, D5> = Mat(D7, D5, Vec(t0, t1, t2, t3, t4, t5, t6))
    @JvmName("m7x6") private operator fun <T, V: Vec<T, D6>> invoke(t0: V, t1: V, t2: V, t3: V, t4: V, t5: V, t6: V): Mat<T, D7, D6> = Mat(D7, D6, Vec(t0, t1, t2, t3, t4, t5, t6))
    @JvmName("m7x7") private operator fun <T, V: Vec<T, D7>> invoke(t0: V, t1: V, t2: V, t3: V, t4: V, t5: V, t6: V): Mat<T, D7, D7> = Mat(D7, D7, Vec(t0, t1, t2, t3, t4, t5, t6))
    @JvmName("m7x8") private operator fun <T, V: Vec<T, D8>> invoke(t0: V, t1: V, t2: V, t3: V, t4: V, t5: V, t6: V): Mat<T, D7, D8> = Mat(D7, D8, Vec(t0, t1, t2, t3, t4, t5, t6))
    @JvmName("m7x9") private operator fun <T, V: Vec<T, D9>> invoke(t0: V, t1: V, t2: V, t3: V, t4: V, t5: V, t6: V): Mat<T, D7, D9> = Mat(D7, D9, Vec(t0, t1, t2, t3, t4, t5, t6))
    @JvmName("m8x1") private operator fun <T, V: Vec<T, D1>> invoke(t0: V, t1: V, t2: V, t3: V, t4: V, t5: V, t6: V, t7: V): Mat<T, D8, D1> = Mat(D8, D1, Vec(t0, t1, t2, t3, t4, t5, t6, t7))
    @JvmName("m8x2") private operator fun <T, V: Vec<T, D2>> invoke(t0: V, t1: V, t2: V, t3: V, t4: V, t5: V, t6: V, t7: V): Mat<T, D8, D2> = Mat(D8, D2, Vec(t0, t1, t2, t3, t4, t5, t6, t7))
    @JvmName("m8x3") private operator fun <T, V: Vec<T, D3>> invoke(t0: V, t1: V, t2: V, t3: V, t4: V, t5: V, t6: V, t7: V): Mat<T, D8, D3> = Mat(D8, D3, Vec(t0, t1, t2, t3, t4, t5, t6, t7))
    @JvmName("m8x4") private operator fun <T, V: Vec<T, D4>> invoke(t0: V, t1: V, t2: V, t3: V, t4: V, t5: V, t6: V, t7: V): Mat<T, D8, D4> = Mat(D8, D4, Vec(t0, t1, t2, t3, t4, t5, t6, t7))
    @JvmName("m8x5") private operator fun <T, V: Vec<T, D5>> invoke(t0: V, t1: V, t2: V, t3: V, t4: V, t5: V, t6: V, t7: V): Mat<T, D8, D5> = Mat(D8, D5, Vec(t0, t1, t2, t3, t4, t5, t6, t7))
    @JvmName("m8x6") private operator fun <T, V: Vec<T, D6>> invoke(t0: V, t1: V, t2: V, t3: V, t4: V, t5: V, t6: V, t7: V): Mat<T, D8, D6> = Mat(D8, D6, Vec(t0, t1, t2, t3, t4, t5, t6, t7))
    @JvmName("m8x7") private operator fun <T, V: Vec<T, D7>> invoke(t0: V, t1: V, t2: V, t3: V, t4: V, t5: V, t6: V, t7: V): Mat<T, D8, D7> = Mat(D8, D7, Vec(t0, t1, t2, t3, t4, t5, t6, t7))
    @JvmName("m8x8") private operator fun <T, V: Vec<T, D8>> invoke(t0: V, t1: V, t2: V, t3: V, t4: V, t5: V, t6: V, t7: V): Mat<T, D8, D8> = Mat(D8, D8, Vec(t0, t1, t2, t3, t4, t5, t6, t7))
    @JvmName("m8x9") private operator fun <T, V: Vec<T, D9>> invoke(t0: V, t1: V, t2: V, t3: V, t4: V, t5: V, t6: V, t7: V): Mat<T, D8, D9> = Mat(D8, D9, Vec(t0, t1, t2, t3, t4, t5, t6, t7))
    @JvmName("m9x1") private operator fun <T, V: Vec<T, D1>> invoke(t0: V, t1: V, t2: V, t3: V, t4: V, t5: V, t6: V, t7: V, t8: V): Mat<T, D9, D1> = Mat(D9, D1, Vec(t0, t1, t2, t3, t4, t5, t6, t7, t8))
    @JvmName("m9x2") private operator fun <T, V: Vec<T, D2>> invoke(t0: V, t1: V, t2: V, t3: V, t4: V, t5: V, t6: V, t7: V, t8: V): Mat<T, D9, D2> = Mat(D9, D2, Vec(t0, t1, t2, t3, t4, t5, t6, t7, t8))
    @JvmName("m9x3") private operator fun <T, V: Vec<T, D3>> invoke(t0: V, t1: V, t2: V, t3: V, t4: V, t5: V, t6: V, t7: V, t8: V): Mat<T, D9, D3> = Mat(D9, D3, Vec(t0, t1, t2, t3, t4, t5, t6, t7, t8))
    @JvmName("m9x4") private operator fun <T, V: Vec<T, D4>> invoke(t0: V, t1: V, t2: V, t3: V, t4: V, t5: V, t6: V, t7: V, t8: V): Mat<T, D9, D4> = Mat(D9, D4, Vec(t0, t1, t2, t3, t4, t5, t6, t7, t8))
    @JvmName("m9x5") private operator fun <T, V: Vec<T, D5>> invoke(t0: V, t1: V, t2: V, t3: V, t4: V, t5: V, t6: V, t7: V, t8: V): Mat<T, D9, D5> = Mat(D9, D5, Vec(t0, t1, t2, t3, t4, t5, t6, t7, t8))
    @JvmName("m9x6") private operator fun <T, V: Vec<T, D6>> invoke(t0: V, t1: V, t2: V, t3: V, t4: V, t5: V, t6: V, t7: V, t8: V): Mat<T, D9, D6> = Mat(D9, D6, Vec(t0, t1, t2, t3, t4, t5, t6, t7, t8))
    @JvmName("m9x7") private operator fun <T, V: Vec<T, D7>> invoke(t0: V, t1: V, t2: V, t3: V, t4: V, t5: V, t6: V, t7: V, t8: V): Mat<T, D9, D7> = Mat(D9, D7, Vec(t0, t1, t2, t3, t4, t5, t6, t7, t8))
    @JvmName("m9x8") private operator fun <T, V: Vec<T, D8>> invoke(t0: V, t1: V, t2: V, t3: V, t4: V, t5: V, t6: V, t7: V, t8: V): Mat<T, D9, D8> = Mat(D9, D8, Vec(t0, t1, t2, t3, t4, t5, t6, t7, t8))
    @JvmName("m9x9") private operator fun <T, V: Vec<T, D9>> invoke(t0: V, t1: V, t2: V, t3: V, t4: V, t5: V, t6: V, t7: V, t8: V): Mat<T, D9, D9> = Mat(D9, D9, Vec(t0, t1, t2, t3, t4, t5, t6, t7, t8))

    @JvmName("m1x2f") operator fun <T> invoke(m: Nat<D1>, n: Nat<D2>, t0: T, t1: T) = Mat(Vec(t0, t1))
    @JvmName("m1x3f") operator fun <T> invoke(m: Nat<D1>, n: Nat<D3>, t0: T, t1: T, t2: T) = Mat(Vec(t0, t1, t2))
    @JvmName("m1x4f") operator fun <T> invoke(m: Nat<D1>, n: Nat<D4>, t0: T, t1: T, t2: T, t3: T) = Mat(Vec(t0, t1, t2, t3))
    @JvmName("m1x5f") operator fun <T> invoke(m: Nat<D1>, n: Nat<D5>, t0: T, t1: T, t2: T, t3: T, t4: T) = Mat(Vec(t0, t1, t2, t3, t4))
    @JvmName("m1x6f") operator fun <T> invoke(m: Nat<D1>, n: Nat<D6>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T) = Mat(Vec(t0, t1, t2, t3, t4, t5))
    @JvmName("m1x7f") operator fun <T> invoke(m: Nat<D1>, n: Nat<D7>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T, t6: T) = Mat(Vec(t0, t1, t2, t3, t4, t5, t6))
    @JvmName("m1x8f") operator fun <T> invoke(m: Nat<D1>, n: Nat<D8>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T, t6: T, t7: T) = Mat(Vec(t0, t1, t2, t3, t4, t5, t6, t7))
    @JvmName("m1x9f") operator fun <T> invoke(m: Nat<D1>, n: Nat<D9>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T, t6: T, t7: T, t8: T) = Mat(Vec(t0, t1, t2, t3, t4, t5, t6, t7, t8))
    @JvmName("m2x1f") operator fun <T> invoke(m: Nat<D2>, n: Nat<D1>, t0: T, t1: T) = Mat(Vec(t0), Vec(t1))
    @JvmName("m2x2f") operator fun <T> invoke(m: Nat<D2>, n: Nat<D2>, t0: T, t1: T, t2: T, t3: T) = Mat(Vec(t0, t1), Vec(t2, t3))
    @JvmName("m2x3f") operator fun <T> invoke(m: Nat<D2>, n: Nat<D3>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T) = Mat(Vec(t0, t1, t2), Vec(t3, t4, t5))
    @JvmName("m2x4f") operator fun <T> invoke(m: Nat<D2>, n: Nat<D4>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T, t6: T, t7: T) = Mat(Vec(t0, t1, t2, t3), Vec(t4, t5, t6, t7))
    @JvmName("m2x5f") operator fun <T> invoke(m: Nat<D2>, n: Nat<D5>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T, t6: T, t7: T, t8: T, t9: T) = Mat(Vec(t0, t1, t2, t3, t4), Vec(t5, t6, t7, t8, t9))
    @JvmName("m2x6f") operator fun <T> invoke(m: Nat<D2>, n: Nat<D6>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T, t6: T, t7: T, t8: T, t9: T, t10: T, t11: T) = Mat(Vec(t0, t1, t2, t3, t4, t5), Vec(t6, t7, t8, t9, t10, t11))
    @JvmName("m2x7f") operator fun <T> invoke(m: Nat<D2>, n: Nat<D7>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T, t6: T, t7: T, t8: T, t9: T, t10: T, t11: T, t12: T, t13: T) = Mat(Vec(t0, t1, t2, t3, t4, t5, t6), Vec(t7, t8, t9, t10, t11, t12, t13))
    @JvmName("m2x8f") operator fun <T> invoke(m: Nat<D2>, n: Nat<D8>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T, t6: T, t7: T, t8: T, t9: T, t10: T, t11: T, t12: T, t13: T, t14: T, t15: T) = Mat(Vec(t0, t1, t2, t3, t4, t5, t6, t7), Vec(t8, t9, t10, t11, t12, t13, t14, t15))
    @JvmName("m2x9f") operator fun <T> invoke(m: Nat<D2>, n: Nat<D9>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T, t6: T, t7: T, t8: T, t9: T, t10: T, t11: T, t12: T, t13: T, t14: T, t15: T, t16: T, t17: T) = Mat(Vec(t0, t1, t2, t3, t4, t5, t6, t7, t8), Vec(t9, t10, t11, t12, t13, t14, t15, t16, t17))
    @JvmName("m3x1f") operator fun <T> invoke(m: Nat<D3>, n: Nat<D1>, t0: T, t1: T, t2: T) = Mat(Vec(t0), Vec(t1), Vec(t2))
    @JvmName("m3x2f") operator fun <T> invoke(m: Nat<D3>, n: Nat<D2>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T) = Mat(Vec(t0, t1), Vec(t2, t3), Vec(t4, t5))
    @JvmName("m3x3f") operator fun <T> invoke(m: Nat<D3>, n: Nat<D3>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T, t6: T, t7: T, t8: T) = Mat(Vec(t0, t1, t2), Vec(t3, t4, t5), Vec(t6, t7, t8))
    @JvmName("m3x4f") operator fun <T> invoke(m: Nat<D3>, n: Nat<D4>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T, t6: T, t7: T, t8: T, t9: T, t10: T, t11: T) = Mat(Vec(t0, t1, t2, t3), Vec(t4, t5, t6, t7), Vec(t8, t9, t10, t11))
    @JvmName("m3x5f") operator fun <T> invoke(m: Nat<D3>, n: Nat<D5>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T, t6: T, t7: T, t8: T, t9: T, t10: T, t11: T, t12: T, t13: T, t14: T) = Mat(Vec(t0, t1, t2, t3, t4), Vec(t5, t6, t7, t8, t9), Vec(t10, t11, t12, t13, t14))
    @JvmName("m3x6f") operator fun <T> invoke(m: Nat<D3>, n: Nat<D6>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T, t6: T, t7: T, t8: T, t9: T, t10: T, t11: T, t12: T, t13: T, t14: T, t15: T, t16: T, t17: T) = Mat(Vec(t0, t1, t2, t3, t4, t5), Vec(t6, t7, t8, t9, t10, t11), Vec(t12, t13, t14, t15, t16, t17))
    @JvmName("m3x7f") operator fun <T> invoke(m: Nat<D3>, n: Nat<D7>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T, t6: T, t7: T, t8: T, t9: T, t10: T, t11: T, t12: T, t13: T, t14: T, t15: T, t16: T, t17: T, t18: T, t19: T, t20: T) = Mat(Vec(t0, t1, t2, t3, t4, t5, t6), Vec(t7, t8, t9, t10, t11, t12, t13), Vec(t14, t15, t16, t17, t18, t19, t20))
    @JvmName("m3x8f") operator fun <T> invoke(m: Nat<D3>, n: Nat<D8>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T, t6: T, t7: T, t8: T, t9: T, t10: T, t11: T, t12: T, t13: T, t14: T, t15: T, t16: T, t17: T, t18: T, t19: T, t20: T, t21: T, t22: T, t23: T) = Mat(Vec(t0, t1, t2, t3, t4, t5, t6, t7), Vec(t8, t9, t10, t11, t12, t13, t14, t15), Vec(t16, t17, t18, t19, t20, t21, t22, t23))
    @JvmName("m3x9f") operator fun <T> invoke(m: Nat<D3>, n: Nat<D9>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T, t6: T, t7: T, t8: T, t9: T, t10: T, t11: T, t12: T, t13: T, t14: T, t15: T, t16: T, t17: T, t18: T, t19: T, t20: T, t21: T, t22: T, t23: T, t24: T, t25: T, t26: T) = Mat(Vec(t0, t1, t2, t3, t4, t5, t6, t7, t8), Vec(t9, t10, t11, t12, t13, t14, t15, t16, t17), Vec(t18, t19, t20, t21, t22, t23, t24, t25, t26))
    @JvmName("m4x1f") operator fun <T> invoke(m: Nat<D4>, n: Nat<D1>, t0: T, t1: T, t2: T, t3: T) = Mat(Vec(t0), Vec(t1), Vec(t2), Vec(t3))
    @JvmName("m4x2f") operator fun <T> invoke(m: Nat<D4>, n: Nat<D2>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T, t6: T, t7: T) = Mat(Vec(t0, t1), Vec(t2, t3), Vec(t4, t5), Vec(t6, t7))
    @JvmName("m4x3f") operator fun <T> invoke(m: Nat<D4>, n: Nat<D3>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T, t6: T, t7: T, t8: T, t9: T, t10: T, t11: T) = Mat(Vec(t0, t1, t2), Vec(t3, t4, t5), Vec(t6, t7, t8), Vec(t9, t10, t11))
    @JvmName("m4x4f") operator fun <T> invoke(m: Nat<D4>, n: Nat<D4>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T, t6: T, t7: T, t8: T, t9: T, t10: T, t11: T, t12: T, t13: T, t14: T, t15: T) = Mat(Vec(t0, t1, t2, t3), Vec(t4, t5, t6, t7), Vec(t8, t9, t10, t11), Vec(t12, t13, t14, t15))
    @JvmName("m4x5f") operator fun <T> invoke(m: Nat<D4>, n: Nat<D5>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T, t6: T, t7: T, t8: T, t9: T, t10: T, t11: T, t12: T, t13: T, t14: T, t15: T, t16: T, t17: T, t18: T, t19: T) = Mat(Vec(t0, t1, t2, t3, t4), Vec(t5, t6, t7, t8, t9), Vec(t10, t11, t12, t13, t14), Vec(t15, t16, t17, t18, t19))
    @JvmName("m4x6f") operator fun <T> invoke(m: Nat<D4>, n: Nat<D6>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T, t6: T, t7: T, t8: T, t9: T, t10: T, t11: T, t12: T, t13: T, t14: T, t15: T, t16: T, t17: T, t18: T, t19: T, t20: T, t21: T, t22: T, t23: T) = Mat(Vec(t0, t1, t2, t3, t4, t5), Vec(t6, t7, t8, t9, t10, t11), Vec(t12, t13, t14, t15, t16, t17), Vec(t18, t19, t20, t21, t22, t23))
    @JvmName("m4x7f") operator fun <T> invoke(m: Nat<D4>, n: Nat<D7>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T, t6: T, t7: T, t8: T, t9: T, t10: T, t11: T, t12: T, t13: T, t14: T, t15: T, t16: T, t17: T, t18: T, t19: T, t20: T, t21: T, t22: T, t23: T, t24: T, t25: T, t26: T, t27: T) = Mat(Vec(t0, t1, t2, t3, t4, t5, t6), Vec(t7, t8, t9, t10, t11, t12, t13), Vec(t14, t15, t16, t17, t18, t19, t20), Vec(t21, t22, t23, t24, t25, t26, t27))
    @JvmName("m4x8f") operator fun <T> invoke(m: Nat<D4>, n: Nat<D8>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T, t6: T, t7: T, t8: T, t9: T, t10: T, t11: T, t12: T, t13: T, t14: T, t15: T, t16: T, t17: T, t18: T, t19: T, t20: T, t21: T, t22: T, t23: T, t24: T, t25: T, t26: T, t27: T, t28: T, t29: T, t30: T, t31: T) = Mat(Vec(t0, t1, t2, t3, t4, t5, t6, t7), Vec(t8, t9, t10, t11, t12, t13, t14, t15), Vec(t16, t17, t18, t19, t20, t21, t22, t23), Vec(t24, t25, t26, t27, t28, t29, t30, t31))
    @JvmName("m4x9f") operator fun <T> invoke(m: Nat<D4>, n: Nat<D9>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T, t6: T, t7: T, t8: T, t9: T, t10: T, t11: T, t12: T, t13: T, t14: T, t15: T, t16: T, t17: T, t18: T, t19: T, t20: T, t21: T, t22: T, t23: T, t24: T, t25: T, t26: T, t27: T, t28: T, t29: T, t30: T, t31: T, t32: T, t33: T, t34: T, t35: T) = Mat(Vec(t0, t1, t2, t3, t4, t5, t6, t7, t8), Vec(t9, t10, t11, t12, t13, t14, t15, t16, t17), Vec(t18, t19, t20, t21, t22, t23, t24, t25, t26), Vec(t27, t28, t29, t30, t31, t32, t33, t34, t35))
    @JvmName("m5x1f") operator fun <T> invoke(m: Nat<D5>, n: Nat<D1>, t0: T, t1: T, t2: T, t3: T, t4: T) = Mat(Vec(t0), Vec(t1), Vec(t2), Vec(t3), Vec(t4))
    @JvmName("m5x2f") operator fun <T> invoke(m: Nat<D5>, n: Nat<D2>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T, t6: T, t7: T, t8: T, t9: T) = Mat(Vec(t0, t1), Vec(t2, t3), Vec(t4, t5), Vec(t6, t7), Vec(t8, t9))
    @JvmName("m5x3f") operator fun <T> invoke(m: Nat<D5>, n: Nat<D3>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T, t6: T, t7: T, t8: T, t9: T, t10: T, t11: T, t12: T, t13: T, t14: T) = Mat(Vec(t0, t1, t2), Vec(t3, t4, t5), Vec(t6, t7, t8), Vec(t9, t10, t11), Vec(t12, t13, t14))
    @JvmName("m5x4f") operator fun <T> invoke(m: Nat<D5>, n: Nat<D4>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T, t6: T, t7: T, t8: T, t9: T, t10: T, t11: T, t12: T, t13: T, t14: T, t15: T, t16: T, t17: T, t18: T, t19: T) = Mat(Vec(t0, t1, t2, t3), Vec(t4, t5, t6, t7), Vec(t8, t9, t10, t11), Vec(t12, t13, t14, t15), Vec(t16, t17, t18, t19))
    @JvmName("m5x5f") operator fun <T> invoke(m: Nat<D5>, n: Nat<D5>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T, t6: T, t7: T, t8: T, t9: T, t10: T, t11: T, t12: T, t13: T, t14: T, t15: T, t16: T, t17: T, t18: T, t19: T, t20: T, t21: T, t22: T, t23: T, t24: T) = Mat(Vec(t0, t1, t2, t3, t4), Vec(t5, t6, t7, t8, t9), Vec(t10, t11, t12, t13, t14), Vec(t15, t16, t17, t18, t19), Vec(t20, t21, t22, t23, t24))
    @JvmName("m5x6f") operator fun <T> invoke(m: Nat<D5>, n: Nat<D6>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T, t6: T, t7: T, t8: T, t9: T, t10: T, t11: T, t12: T, t13: T, t14: T, t15: T, t16: T, t17: T, t18: T, t19: T, t20: T, t21: T, t22: T, t23: T, t24: T, t25: T, t26: T, t27: T, t28: T, t29: T) = Mat(Vec(t0, t1, t2, t3, t4, t5), Vec(t6, t7, t8, t9, t10, t11), Vec(t12, t13, t14, t15, t16, t17), Vec(t18, t19, t20, t21, t22, t23), Vec(t24, t25, t26, t27, t28, t29))
    @JvmName("m5x7f") operator fun <T> invoke(m: Nat<D5>, n: Nat<D7>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T, t6: T, t7: T, t8: T, t9: T, t10: T, t11: T, t12: T, t13: T, t14: T, t15: T, t16: T, t17: T, t18: T, t19: T, t20: T, t21: T, t22: T, t23: T, t24: T, t25: T, t26: T, t27: T, t28: T, t29: T, t30: T, t31: T, t32: T, t33: T, t34: T) = Mat(Vec(t0, t1, t2, t3, t4, t5, t6), Vec(t7, t8, t9, t10, t11, t12, t13), Vec(t14, t15, t16, t17, t18, t19, t20), Vec(t21, t22, t23, t24, t25, t26, t27), Vec(t28, t29, t30, t31, t32, t33, t34))
    @JvmName("m5x8f") operator fun <T> invoke(m: Nat<D5>, n: Nat<D8>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T, t6: T, t7: T, t8: T, t9: T, t10: T, t11: T, t12: T, t13: T, t14: T, t15: T, t16: T, t17: T, t18: T, t19: T, t20: T, t21: T, t22: T, t23: T, t24: T, t25: T, t26: T, t27: T, t28: T, t29: T, t30: T, t31: T, t32: T, t33: T, t34: T, t35: T, t36: T, t37: T, t38: T, t39: T) = Mat(Vec(t0, t1, t2, t3, t4, t5, t6, t7), Vec(t8, t9, t10, t11, t12, t13, t14, t15), Vec(t16, t17, t18, t19, t20, t21, t22, t23), Vec(t24, t25, t26, t27, t28, t29, t30, t31), Vec(t32, t33, t34, t35, t36, t37, t38, t39))
    @JvmName("m5x9f") operator fun <T> invoke(m: Nat<D5>, n: Nat<D9>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T, t6: T, t7: T, t8: T, t9: T, t10: T, t11: T, t12: T, t13: T, t14: T, t15: T, t16: T, t17: T, t18: T, t19: T, t20: T, t21: T, t22: T, t23: T, t24: T, t25: T, t26: T, t27: T, t28: T, t29: T, t30: T, t31: T, t32: T, t33: T, t34: T, t35: T, t36: T, t37: T, t38: T, t39: T, t40: T, t41: T, t42: T, t43: T, t44: T) = Mat(Vec(t0, t1, t2, t3, t4, t5, t6, t7, t8), Vec(t9, t10, t11, t12, t13, t14, t15, t16, t17), Vec(t18, t19, t20, t21, t22, t23, t24, t25, t26), Vec(t27, t28, t29, t30, t31, t32, t33, t34, t35), Vec(t36, t37, t38, t39, t40, t41, t42, t43, t44))
    @JvmName("m6x1f") operator fun <T> invoke(m: Nat<D6>, n: Nat<D1>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T) = Mat(Vec(t0), Vec(t1), Vec(t2), Vec(t3), Vec(t4), Vec(t5))
    @JvmName("m6x2f") operator fun <T> invoke(m: Nat<D6>, n: Nat<D2>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T, t6: T, t7: T, t8: T, t9: T, t10: T, t11: T) = Mat(Vec(t0, t1), Vec(t2, t3), Vec(t4, t5), Vec(t6, t7), Vec(t8, t9), Vec(t10, t11))
    @JvmName("m6x3f") operator fun <T> invoke(m: Nat<D6>, n: Nat<D3>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T, t6: T, t7: T, t8: T, t9: T, t10: T, t11: T, t12: T, t13: T, t14: T, t15: T, t16: T, t17: T) = Mat(Vec(t0, t1, t2), Vec(t3, t4, t5), Vec(t6, t7, t8), Vec(t9, t10, t11), Vec(t12, t13, t14), Vec(t15, t16, t17))
    @JvmName("m6x4f") operator fun <T> invoke(m: Nat<D6>, n: Nat<D4>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T, t6: T, t7: T, t8: T, t9: T, t10: T, t11: T, t12: T, t13: T, t14: T, t15: T, t16: T, t17: T, t18: T, t19: T, t20: T, t21: T, t22: T, t23: T) = Mat(Vec(t0, t1, t2, t3), Vec(t4, t5, t6, t7), Vec(t8, t9, t10, t11), Vec(t12, t13, t14, t15), Vec(t16, t17, t18, t19), Vec(t20, t21, t22, t23))
    @JvmName("m6x5f") operator fun <T> invoke(m: Nat<D6>, n: Nat<D5>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T, t6: T, t7: T, t8: T, t9: T, t10: T, t11: T, t12: T, t13: T, t14: T, t15: T, t16: T, t17: T, t18: T, t19: T, t20: T, t21: T, t22: T, t23: T, t24: T, t25: T, t26: T, t27: T, t28: T, t29: T) = Mat(Vec(t0, t1, t2, t3, t4), Vec(t5, t6, t7, t8, t9), Vec(t10, t11, t12, t13, t14), Vec(t15, t16, t17, t18, t19), Vec(t20, t21, t22, t23, t24), Vec(t25, t26, t27, t28, t29))
    @JvmName("m6x6f") operator fun <T> invoke(m: Nat<D6>, n: Nat<D6>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T, t6: T, t7: T, t8: T, t9: T, t10: T, t11: T, t12: T, t13: T, t14: T, t15: T, t16: T, t17: T, t18: T, t19: T, t20: T, t21: T, t22: T, t23: T, t24: T, t25: T, t26: T, t27: T, t28: T, t29: T, t30: T, t31: T, t32: T, t33: T, t34: T, t35: T) = Mat(Vec(t0, t1, t2, t3, t4, t5), Vec(t6, t7, t8, t9, t10, t11), Vec(t12, t13, t14, t15, t16, t17), Vec(t18, t19, t20, t21, t22, t23), Vec(t24, t25, t26, t27, t28, t29), Vec(t30, t31, t32, t33, t34, t35))
    @JvmName("m6x7f") operator fun <T> invoke(m: Nat<D6>, n: Nat<D7>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T, t6: T, t7: T, t8: T, t9: T, t10: T, t11: T, t12: T, t13: T, t14: T, t15: T, t16: T, t17: T, t18: T, t19: T, t20: T, t21: T, t22: T, t23: T, t24: T, t25: T, t26: T, t27: T, t28: T, t29: T, t30: T, t31: T, t32: T, t33: T, t34: T, t35: T, t36: T, t37: T, t38: T, t39: T, t40: T, t41: T) = Mat(Vec(t0, t1, t2, t3, t4, t5, t6), Vec(t7, t8, t9, t10, t11, t12, t13), Vec(t14, t15, t16, t17, t18, t19, t20), Vec(t21, t22, t23, t24, t25, t26, t27), Vec(t28, t29, t30, t31, t32, t33, t34), Vec(t35, t36, t37, t38, t39, t40, t41))
    @JvmName("m6x8f") operator fun <T> invoke(m: Nat<D6>, n: Nat<D8>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T, t6: T, t7: T, t8: T, t9: T, t10: T, t11: T, t12: T, t13: T, t14: T, t15: T, t16: T, t17: T, t18: T, t19: T, t20: T, t21: T, t22: T, t23: T, t24: T, t25: T, t26: T, t27: T, t28: T, t29: T, t30: T, t31: T, t32: T, t33: T, t34: T, t35: T, t36: T, t37: T, t38: T, t39: T, t40: T, t41: T, t42: T, t43: T, t44: T, t45: T, t46: T, t47: T) = Mat(Vec(t0, t1, t2, t3, t4, t5, t6, t7), Vec(t8, t9, t10, t11, t12, t13, t14, t15), Vec(t16, t17, t18, t19, t20, t21, t22, t23), Vec(t24, t25, t26, t27, t28, t29, t30, t31), Vec(t32, t33, t34, t35, t36, t37, t38, t39), Vec(t40, t41, t42, t43, t44, t45, t46, t47))
    @JvmName("m6x9f") operator fun <T> invoke(m: Nat<D6>, n: Nat<D9>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T, t6: T, t7: T, t8: T, t9: T, t10: T, t11: T, t12: T, t13: T, t14: T, t15: T, t16: T, t17: T, t18: T, t19: T, t20: T, t21: T, t22: T, t23: T, t24: T, t25: T, t26: T, t27: T, t28: T, t29: T, t30: T, t31: T, t32: T, t33: T, t34: T, t35: T, t36: T, t37: T, t38: T, t39: T, t40: T, t41: T, t42: T, t43: T, t44: T, t45: T, t46: T, t47: T, t48: T, t49: T, t50: T, t51: T, t52: T, t53: T) = Mat(Vec(t0, t1, t2, t3, t4, t5, t6, t7, t8), Vec(t9, t10, t11, t12, t13, t14, t15, t16, t17), Vec(t18, t19, t20, t21, t22, t23, t24, t25, t26), Vec(t27, t28, t29, t30, t31, t32, t33, t34, t35), Vec(t36, t37, t38, t39, t40, t41, t42, t43, t44), Vec(t45, t46, t47, t48, t49, t50, t51, t52, t53))
    @JvmName("m7x1f") operator fun <T> invoke(m: Nat<D7>, n: Nat<D1>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T, t6: T) = Mat(Vec(t0), Vec(t1), Vec(t2), Vec(t3), Vec(t4), Vec(t5), Vec(t6))
    @JvmName("m7x2f") operator fun <T> invoke(m: Nat<D7>, n: Nat<D2>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T, t6: T, t7: T, t8: T, t9: T, t10: T, t11: T, t12: T, t13: T) = Mat(Vec(t0, t1), Vec(t2, t3), Vec(t4, t5), Vec(t6, t7), Vec(t8, t9), Vec(t10, t11), Vec(t12, t13))
    @JvmName("m7x3f") operator fun <T> invoke(m: Nat<D7>, n: Nat<D3>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T, t6: T, t7: T, t8: T, t9: T, t10: T, t11: T, t12: T, t13: T, t14: T, t15: T, t16: T, t17: T, t18: T, t19: T, t20: T) = Mat(Vec(t0, t1, t2), Vec(t3, t4, t5), Vec(t6, t7, t8), Vec(t9, t10, t11), Vec(t12, t13, t14), Vec(t15, t16, t17), Vec(t18, t19, t20))
    @JvmName("m7x4f") operator fun <T> invoke(m: Nat<D7>, n: Nat<D4>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T, t6: T, t7: T, t8: T, t9: T, t10: T, t11: T, t12: T, t13: T, t14: T, t15: T, t16: T, t17: T, t18: T, t19: T, t20: T, t21: T, t22: T, t23: T, t24: T, t25: T, t26: T, t27: T) = Mat(Vec(t0, t1, t2, t3), Vec(t4, t5, t6, t7), Vec(t8, t9, t10, t11), Vec(t12, t13, t14, t15), Vec(t16, t17, t18, t19), Vec(t20, t21, t22, t23), Vec(t24, t25, t26, t27))
    @JvmName("m7x5f") operator fun <T> invoke(m: Nat<D7>, n: Nat<D5>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T, t6: T, t7: T, t8: T, t9: T, t10: T, t11: T, t12: T, t13: T, t14: T, t15: T, t16: T, t17: T, t18: T, t19: T, t20: T, t21: T, t22: T, t23: T, t24: T, t25: T, t26: T, t27: T, t28: T, t29: T, t30: T, t31: T, t32: T, t33: T, t34: T) = Mat(Vec(t0, t1, t2, t3, t4), Vec(t5, t6, t7, t8, t9), Vec(t10, t11, t12, t13, t14), Vec(t15, t16, t17, t18, t19), Vec(t20, t21, t22, t23, t24), Vec(t25, t26, t27, t28, t29), Vec(t30, t31, t32, t33, t34))
    @JvmName("m7x6f") operator fun <T> invoke(m: Nat<D7>, n: Nat<D6>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T, t6: T, t7: T, t8: T, t9: T, t10: T, t11: T, t12: T, t13: T, t14: T, t15: T, t16: T, t17: T, t18: T, t19: T, t20: T, t21: T, t22: T, t23: T, t24: T, t25: T, t26: T, t27: T, t28: T, t29: T, t30: T, t31: T, t32: T, t33: T, t34: T, t35: T, t36: T, t37: T, t38: T, t39: T, t40: T, t41: T) = Mat(Vec(t0, t1, t2, t3, t4, t5), Vec(t6, t7, t8, t9, t10, t11), Vec(t12, t13, t14, t15, t16, t17), Vec(t18, t19, t20, t21, t22, t23), Vec(t24, t25, t26, t27, t28, t29), Vec(t30, t31, t32, t33, t34, t35), Vec(t36, t37, t38, t39, t40, t41))
    @JvmName("m7x7f") operator fun <T> invoke(m: Nat<D7>, n: Nat<D7>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T, t6: T, t7: T, t8: T, t9: T, t10: T, t11: T, t12: T, t13: T, t14: T, t15: T, t16: T, t17: T, t18: T, t19: T, t20: T, t21: T, t22: T, t23: T, t24: T, t25: T, t26: T, t27: T, t28: T, t29: T, t30: T, t31: T, t32: T, t33: T, t34: T, t35: T, t36: T, t37: T, t38: T, t39: T, t40: T, t41: T, t42: T, t43: T, t44: T, t45: T, t46: T, t47: T, t48: T) = Mat(Vec(t0, t1, t2, t3, t4, t5, t6), Vec(t7, t8, t9, t10, t11, t12, t13), Vec(t14, t15, t16, t17, t18, t19, t20), Vec(t21, t22, t23, t24, t25, t26, t27), Vec(t28, t29, t30, t31, t32, t33, t34), Vec(t35, t36, t37, t38, t39, t40, t41), Vec(t42, t43, t44, t45, t46, t47, t48))
    @JvmName("m7x8f") operator fun <T> invoke(m: Nat<D7>, n: Nat<D8>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T, t6: T, t7: T, t8: T, t9: T, t10: T, t11: T, t12: T, t13: T, t14: T, t15: T, t16: T, t17: T, t18: T, t19: T, t20: T, t21: T, t22: T, t23: T, t24: T, t25: T, t26: T, t27: T, t28: T, t29: T, t30: T, t31: T, t32: T, t33: T, t34: T, t35: T, t36: T, t37: T, t38: T, t39: T, t40: T, t41: T, t42: T, t43: T, t44: T, t45: T, t46: T, t47: T, t48: T, t49: T, t50: T, t51: T, t52: T, t53: T, t54: T, t55: T) = Mat(Vec(t0, t1, t2, t3, t4, t5, t6, t7), Vec(t8, t9, t10, t11, t12, t13, t14, t15), Vec(t16, t17, t18, t19, t20, t21, t22, t23), Vec(t24, t25, t26, t27, t28, t29, t30, t31), Vec(t32, t33, t34, t35, t36, t37, t38, t39), Vec(t40, t41, t42, t43, t44, t45, t46, t47), Vec(t48, t49, t50, t51, t52, t53, t54, t55))
    @JvmName("m7x9f") operator fun <T> invoke(m: Nat<D7>, n: Nat<D9>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T, t6: T, t7: T, t8: T, t9: T, t10: T, t11: T, t12: T, t13: T, t14: T, t15: T, t16: T, t17: T, t18: T, t19: T, t20: T, t21: T, t22: T, t23: T, t24: T, t25: T, t26: T, t27: T, t28: T, t29: T, t30: T, t31: T, t32: T, t33: T, t34: T, t35: T, t36: T, t37: T, t38: T, t39: T, t40: T, t41: T, t42: T, t43: T, t44: T, t45: T, t46: T, t47: T, t48: T, t49: T, t50: T, t51: T, t52: T, t53: T, t54: T, t55: T, t56: T, t57: T, t58: T, t59: T, t60: T, t61: T, t62: T) = Mat(Vec(t0, t1, t2, t3, t4, t5, t6, t7, t8), Vec(t9, t10, t11, t12, t13, t14, t15, t16, t17), Vec(t18, t19, t20, t21, t22, t23, t24, t25, t26), Vec(t27, t28, t29, t30, t31, t32, t33, t34, t35), Vec(t36, t37, t38, t39, t40, t41, t42, t43, t44), Vec(t45, t46, t47, t48, t49, t50, t51, t52, t53), Vec(t54, t55, t56, t57, t58, t59, t60, t61, t62))
    @JvmName("m8x1f") operator fun <T> invoke(m: Nat<D8>, n: Nat<D1>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T, t6: T, t7: T) = Mat(Vec(t0), Vec(t1), Vec(t2), Vec(t3), Vec(t4), Vec(t5), Vec(t6), Vec(t7))
    @JvmName("m8x2f") operator fun <T> invoke(m: Nat<D8>, n: Nat<D2>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T, t6: T, t7: T, t8: T, t9: T, t10: T, t11: T, t12: T, t13: T, t14: T, t15: T) = Mat(Vec(t0, t1), Vec(t2, t3), Vec(t4, t5), Vec(t6, t7), Vec(t8, t9), Vec(t10, t11), Vec(t12, t13), Vec(t14, t15))
    @JvmName("m8x3f") operator fun <T> invoke(m: Nat<D8>, n: Nat<D3>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T, t6: T, t7: T, t8: T, t9: T, t10: T, t11: T, t12: T, t13: T, t14: T, t15: T, t16: T, t17: T, t18: T, t19: T, t20: T, t21: T, t22: T, t23: T) = Mat(Vec(t0, t1, t2), Vec(t3, t4, t5), Vec(t6, t7, t8), Vec(t9, t10, t11), Vec(t12, t13, t14), Vec(t15, t16, t17), Vec(t18, t19, t20), Vec(t21, t22, t23))
    @JvmName("m8x4f") operator fun <T> invoke(m: Nat<D8>, n: Nat<D4>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T, t6: T, t7: T, t8: T, t9: T, t10: T, t11: T, t12: T, t13: T, t14: T, t15: T, t16: T, t17: T, t18: T, t19: T, t20: T, t21: T, t22: T, t23: T, t24: T, t25: T, t26: T, t27: T, t28: T, t29: T, t30: T, t31: T) = Mat(Vec(t0, t1, t2, t3), Vec(t4, t5, t6, t7), Vec(t8, t9, t10, t11), Vec(t12, t13, t14, t15), Vec(t16, t17, t18, t19), Vec(t20, t21, t22, t23), Vec(t24, t25, t26, t27), Vec(t28, t29, t30, t31))
    @JvmName("m8x5f") operator fun <T> invoke(m: Nat<D8>, n: Nat<D5>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T, t6: T, t7: T, t8: T, t9: T, t10: T, t11: T, t12: T, t13: T, t14: T, t15: T, t16: T, t17: T, t18: T, t19: T, t20: T, t21: T, t22: T, t23: T, t24: T, t25: T, t26: T, t27: T, t28: T, t29: T, t30: T, t31: T, t32: T, t33: T, t34: T, t35: T, t36: T, t37: T, t38: T, t39: T) = Mat(Vec(t0, t1, t2, t3, t4), Vec(t5, t6, t7, t8, t9), Vec(t10, t11, t12, t13, t14), Vec(t15, t16, t17, t18, t19), Vec(t20, t21, t22, t23, t24), Vec(t25, t26, t27, t28, t29), Vec(t30, t31, t32, t33, t34), Vec(t35, t36, t37, t38, t39))
    @JvmName("m8x6f") operator fun <T> invoke(m: Nat<D8>, n: Nat<D6>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T, t6: T, t7: T, t8: T, t9: T, t10: T, t11: T, t12: T, t13: T, t14: T, t15: T, t16: T, t17: T, t18: T, t19: T, t20: T, t21: T, t22: T, t23: T, t24: T, t25: T, t26: T, t27: T, t28: T, t29: T, t30: T, t31: T, t32: T, t33: T, t34: T, t35: T, t36: T, t37: T, t38: T, t39: T, t40: T, t41: T, t42: T, t43: T, t44: T, t45: T, t46: T, t47: T) = Mat(Vec(t0, t1, t2, t3, t4, t5), Vec(t6, t7, t8, t9, t10, t11), Vec(t12, t13, t14, t15, t16, t17), Vec(t18, t19, t20, t21, t22, t23), Vec(t24, t25, t26, t27, t28, t29), Vec(t30, t31, t32, t33, t34, t35), Vec(t36, t37, t38, t39, t40, t41), Vec(t42, t43, t44, t45, t46, t47))
    @JvmName("m8x7f") operator fun <T> invoke(m: Nat<D8>, n: Nat<D7>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T, t6: T, t7: T, t8: T, t9: T, t10: T, t11: T, t12: T, t13: T, t14: T, t15: T, t16: T, t17: T, t18: T, t19: T, t20: T, t21: T, t22: T, t23: T, t24: T, t25: T, t26: T, t27: T, t28: T, t29: T, t30: T, t31: T, t32: T, t33: T, t34: T, t35: T, t36: T, t37: T, t38: T, t39: T, t40: T, t41: T, t42: T, t43: T, t44: T, t45: T, t46: T, t47: T, t48: T, t49: T, t50: T, t51: T, t52: T, t53: T, t54: T, t55: T) = Mat(Vec(t0, t1, t2, t3, t4, t5, t6), Vec(t7, t8, t9, t10, t11, t12, t13), Vec(t14, t15, t16, t17, t18, t19, t20), Vec(t21, t22, t23, t24, t25, t26, t27), Vec(t28, t29, t30, t31, t32, t33, t34), Vec(t35, t36, t37, t38, t39, t40, t41), Vec(t42, t43, t44, t45, t46, t47, t48), Vec(t49, t50, t51, t52, t53, t54, t55))
    @JvmName("m8x8f") operator fun <T> invoke(m: Nat<D8>, n: Nat<D8>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T, t6: T, t7: T, t8: T, t9: T, t10: T, t11: T, t12: T, t13: T, t14: T, t15: T, t16: T, t17: T, t18: T, t19: T, t20: T, t21: T, t22: T, t23: T, t24: T, t25: T, t26: T, t27: T, t28: T, t29: T, t30: T, t31: T, t32: T, t33: T, t34: T, t35: T, t36: T, t37: T, t38: T, t39: T, t40: T, t41: T, t42: T, t43: T, t44: T, t45: T, t46: T, t47: T, t48: T, t49: T, t50: T, t51: T, t52: T, t53: T, t54: T, t55: T, t56: T, t57: T, t58: T, t59: T, t60: T, t61: T, t62: T, t63: T) = Mat(Vec(t0, t1, t2, t3, t4, t5, t6, t7), Vec(t8, t9, t10, t11, t12, t13, t14, t15), Vec(t16, t17, t18, t19, t20, t21, t22, t23), Vec(t24, t25, t26, t27, t28, t29, t30, t31), Vec(t32, t33, t34, t35, t36, t37, t38, t39), Vec(t40, t41, t42, t43, t44, t45, t46, t47), Vec(t48, t49, t50, t51, t52, t53, t54, t55), Vec(t56, t57, t58, t59, t60, t61, t62, t63))
    @JvmName("m8x9f") operator fun <T> invoke(m: Nat<D8>, n: Nat<D9>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T, t6: T, t7: T, t8: T, t9: T, t10: T, t11: T, t12: T, t13: T, t14: T, t15: T, t16: T, t17: T, t18: T, t19: T, t20: T, t21: T, t22: T, t23: T, t24: T, t25: T, t26: T, t27: T, t28: T, t29: T, t30: T, t31: T, t32: T, t33: T, t34: T, t35: T, t36: T, t37: T, t38: T, t39: T, t40: T, t41: T, t42: T, t43: T, t44: T, t45: T, t46: T, t47: T, t48: T, t49: T, t50: T, t51: T, t52: T, t53: T, t54: T, t55: T, t56: T, t57: T, t58: T, t59: T, t60: T, t61: T, t62: T, t63: T, t64: T, t65: T, t66: T, t67: T, t68: T, t69: T, t70: T, t71: T) = Mat(Vec(t0, t1, t2, t3, t4, t5, t6, t7, t8), Vec(t9, t10, t11, t12, t13, t14, t15, t16, t17), Vec(t18, t19, t20, t21, t22, t23, t24, t25, t26), Vec(t27, t28, t29, t30, t31, t32, t33, t34, t35), Vec(t36, t37, t38, t39, t40, t41, t42, t43, t44), Vec(t45, t46, t47, t48, t49, t50, t51, t52, t53), Vec(t54, t55, t56, t57, t58, t59, t60, t61, t62), Vec(t63, t64, t65, t66, t67, t68, t69, t70, t71))
    @JvmName("m9x1f") operator fun <T> invoke(m: Nat<D9>, n: Nat<D1>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T, t6: T, t7: T, t8: T) = Mat(Vec(t0), Vec(t1), Vec(t2), Vec(t3), Vec(t4), Vec(t5), Vec(t6), Vec(t7), Vec(t8))
    @JvmName("m9x2f") operator fun <T> invoke(m: Nat<D9>, n: Nat<D2>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T, t6: T, t7: T, t8: T, t9: T, t10: T, t11: T, t12: T, t13: T, t14: T, t15: T, t16: T, t17: T) = Mat(Vec(t0, t1), Vec(t2, t3), Vec(t4, t5), Vec(t6, t7), Vec(t8, t9), Vec(t10, t11), Vec(t12, t13), Vec(t14, t15), Vec(t16, t17))
    @JvmName("m9x3f") operator fun <T> invoke(m: Nat<D9>, n: Nat<D3>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T, t6: T, t7: T, t8: T, t9: T, t10: T, t11: T, t12: T, t13: T, t14: T, t15: T, t16: T, t17: T, t18: T, t19: T, t20: T, t21: T, t22: T, t23: T, t24: T, t25: T, t26: T) = Mat(Vec(t0, t1, t2), Vec(t3, t4, t5), Vec(t6, t7, t8), Vec(t9, t10, t11), Vec(t12, t13, t14), Vec(t15, t16, t17), Vec(t18, t19, t20), Vec(t21, t22, t23), Vec(t24, t25, t26))
    @JvmName("m9x4f") operator fun <T> invoke(m: Nat<D9>, n: Nat<D4>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T, t6: T, t7: T, t8: T, t9: T, t10: T, t11: T, t12: T, t13: T, t14: T, t15: T, t16: T, t17: T, t18: T, t19: T, t20: T, t21: T, t22: T, t23: T, t24: T, t25: T, t26: T, t27: T, t28: T, t29: T, t30: T, t31: T, t32: T, t33: T, t34: T, t35: T) = Mat(Vec(t0, t1, t2, t3), Vec(t4, t5, t6, t7), Vec(t8, t9, t10, t11), Vec(t12, t13, t14, t15), Vec(t16, t17, t18, t19), Vec(t20, t21, t22, t23), Vec(t24, t25, t26, t27), Vec(t28, t29, t30, t31), Vec(t32, t33, t34, t35))
    @JvmName("m9x5f") operator fun <T> invoke(m: Nat<D9>, n: Nat<D5>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T, t6: T, t7: T, t8: T, t9: T, t10: T, t11: T, t12: T, t13: T, t14: T, t15: T, t16: T, t17: T, t18: T, t19: T, t20: T, t21: T, t22: T, t23: T, t24: T, t25: T, t26: T, t27: T, t28: T, t29: T, t30: T, t31: T, t32: T, t33: T, t34: T, t35: T, t36: T, t37: T, t38: T, t39: T, t40: T, t41: T, t42: T, t43: T, t44: T) = Mat(Vec(t0, t1, t2, t3, t4), Vec(t5, t6, t7, t8, t9), Vec(t10, t11, t12, t13, t14), Vec(t15, t16, t17, t18, t19), Vec(t20, t21, t22, t23, t24), Vec(t25, t26, t27, t28, t29), Vec(t30, t31, t32, t33, t34), Vec(t35, t36, t37, t38, t39), Vec(t40, t41, t42, t43, t44))
    @JvmName("m9x6f") operator fun <T> invoke(m: Nat<D9>, n: Nat<D6>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T, t6: T, t7: T, t8: T, t9: T, t10: T, t11: T, t12: T, t13: T, t14: T, t15: T, t16: T, t17: T, t18: T, t19: T, t20: T, t21: T, t22: T, t23: T, t24: T, t25: T, t26: T, t27: T, t28: T, t29: T, t30: T, t31: T, t32: T, t33: T, t34: T, t35: T, t36: T, t37: T, t38: T, t39: T, t40: T, t41: T, t42: T, t43: T, t44: T, t45: T, t46: T, t47: T, t48: T, t49: T, t50: T, t51: T, t52: T, t53: T) = Mat(Vec(t0, t1, t2, t3, t4, t5), Vec(t6, t7, t8, t9, t10, t11), Vec(t12, t13, t14, t15, t16, t17), Vec(t18, t19, t20, t21, t22, t23), Vec(t24, t25, t26, t27, t28, t29), Vec(t30, t31, t32, t33, t34, t35), Vec(t36, t37, t38, t39, t40, t41), Vec(t42, t43, t44, t45, t46, t47), Vec(t48, t49, t50, t51, t52, t53))
    @JvmName("m9x7f") operator fun <T> invoke(m: Nat<D9>, n: Nat<D7>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T, t6: T, t7: T, t8: T, t9: T, t10: T, t11: T, t12: T, t13: T, t14: T, t15: T, t16: T, t17: T, t18: T, t19: T, t20: T, t21: T, t22: T, t23: T, t24: T, t25: T, t26: T, t27: T, t28: T, t29: T, t30: T, t31: T, t32: T, t33: T, t34: T, t35: T, t36: T, t37: T, t38: T, t39: T, t40: T, t41: T, t42: T, t43: T, t44: T, t45: T, t46: T, t47: T, t48: T, t49: T, t50: T, t51: T, t52: T, t53: T, t54: T, t55: T, t56: T, t57: T, t58: T, t59: T, t60: T, t61: T, t62: T) = Mat(Vec(t0, t1, t2, t3, t4, t5, t6), Vec(t7, t8, t9, t10, t11, t12, t13), Vec(t14, t15, t16, t17, t18, t19, t20), Vec(t21, t22, t23, t24, t25, t26, t27), Vec(t28, t29, t30, t31, t32, t33, t34), Vec(t35, t36, t37, t38, t39, t40, t41), Vec(t42, t43, t44, t45, t46, t47, t48), Vec(t49, t50, t51, t52, t53, t54, t55), Vec(t56, t57, t58, t59, t60, t61, t62))
    @JvmName("m9x8f") operator fun <T> invoke(m: Nat<D9>, n: Nat<D8>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T, t6: T, t7: T, t8: T, t9: T, t10: T, t11: T, t12: T, t13: T, t14: T, t15: T, t16: T, t17: T, t18: T, t19: T, t20: T, t21: T, t22: T, t23: T, t24: T, t25: T, t26: T, t27: T, t28: T, t29: T, t30: T, t31: T, t32: T, t33: T, t34: T, t35: T, t36: T, t37: T, t38: T, t39: T, t40: T, t41: T, t42: T, t43: T, t44: T, t45: T, t46: T, t47: T, t48: T, t49: T, t50: T, t51: T, t52: T, t53: T, t54: T, t55: T, t56: T, t57: T, t58: T, t59: T, t60: T, t61: T, t62: T, t63: T, t64: T, t65: T, t66: T, t67: T, t68: T, t69: T, t70: T, t71: T) = Mat(Vec(t0, t1, t2, t3, t4, t5, t6, t7), Vec(t8, t9, t10, t11, t12, t13, t14, t15), Vec(t16, t17, t18, t19, t20, t21, t22, t23), Vec(t24, t25, t26, t27, t28, t29, t30, t31), Vec(t32, t33, t34, t35, t36, t37, t38, t39), Vec(t40, t41, t42, t43, t44, t45, t46, t47), Vec(t48, t49, t50, t51, t52, t53, t54, t55), Vec(t56, t57, t58, t59, t60, t61, t62, t63), Vec(t64, t65, t66, t67, t68, t69, t70, t71))
    @JvmName("m9x9f") operator fun <T> invoke(m: Nat<D9>, n: Nat<D9>, t0: T, t1: T, t2: T, t3: T, t4: T, t5: T, t6: T, t7: T, t8: T, t9: T, t10: T, t11: T, t12: T, t13: T, t14: T, t15: T, t16: T, t17: T, t18: T, t19: T, t20: T, t21: T, t22: T, t23: T, t24: T, t25: T, t26: T, t27: T, t28: T, t29: T, t30: T, t31: T, t32: T, t33: T, t34: T, t35: T, t36: T, t37: T, t38: T, t39: T, t40: T, t41: T, t42: T, t43: T, t44: T, t45: T, t46: T, t47: T, t48: T, t49: T, t50: T, t51: T, t52: T, t53: T, t54: T, t55: T, t56: T, t57: T, t58: T, t59: T, t60: T, t61: T, t62: T, t63: T, t64: T, t65: T, t66: T, t67: T, t68: T, t69: T, t70: T, t71: T, t72: T, t73: T, t74: T, t75: T, t76: T, t77: T, t78: T, t79: T, t80: T) = Mat(Vec(t0, t1, t2, t3, t4, t5, t6, t7, t8), Vec(t9, t10, t11, t12, t13, t14, t15, t16, t17), Vec(t18, t19, t20, t21, t22, t23, t24, t25, t26), Vec(t27, t28, t29, t30, t31, t32, t33, t34, t35), Vec(t36, t37, t38, t39, t40, t41, t42, t43, t44), Vec(t45, t46, t47, t48, t49, t50, t51, t52, t53), Vec(t54, t55, t56, t57, t58, t59, t60, t61, t62), Vec(t63, t64, t65, t66, t67, t68, t69, t70, t71), Vec(t72, t73, t74, t75, t76, t77, t78, t79, t80))
  }
}